@@ -37,3 +37,11 @@ plugin code to further minimise dependancies.
 """
 
 __docformat__ = "markdown"  # explicitly disable rST processing in the examples above.
+
+# if we're running inside Snowflake, we want to default the snowflake logging to WARN
+# so that we don't get a ton of snowflake queries in the logs
+import sys
+import logging
+if "snowflake_import_directory" in sys._xoptions:
+    logger = logging.getLogger("snowflake")
+    logger.setLevel(logging.WARN)  # we don't want snowflake queries being logged by default
