@@ -66,8 +66,9 @@ class PluginMessageStreamProgressUpdate(BaseModel):
     message_type: Literal["stream_record_counts"] = "stream_record_counts"
     stream_total_counts: Dict[str, int]
     completed_streams: List[str]
-    # older runtime versions didn't have this, so the sync engine can't expect it
-    stream_errors: Optional[Dict[str,str]]
+    # older runtime versions didn't have these, so the sync engine can't expect it
+    stream_errors: Optional[Dict[str,str]] = None
+    total_records_estimate: Optional[Dict[str,int]] = None
 
 
 class PluginMessageCancelledStreams(BaseModel):
@@ -119,14 +120,15 @@ class OutboundSyncRequestPayload(BaseModel):
     logging_level: str
     connection_method: str
     connection_parameters: Dict[str, StoredConfigurationValue]
-    oauth_secret_name: Optional[str]
-    other_secrets_name: Optional[str]
+    oauth_secret_name: Optional[str] = None
+    other_secrets_name: Optional[str] = None
     sync_direction: Literal["outbound"] = "outbound"
     sync_strategy: OutboundSyncStrategy
     sync_parameters: Dict[str, StoredConfigurationValue]
     api_limit_overrides: List[ApiLimits]
     rate_limits_state: Dict[int,Dict[str,Dict[str,RateLimitState]]]
-    field_mappings: Optional[StoredMappingValue]
+    field_mappings: Optional[StoredMappingValue] = None
+    time_limit_mins: int = 60 * 4
 
 
 class InboundSyncRequestPayload(BaseModel):
@@ -136,7 +138,7 @@ class InboundSyncRequestPayload(BaseModel):
 
     sync_id: int  # only used by log handler
     sync_branch_name: str = 'main' # only used by rate limit updater
-    sync_branch_id: Optional[int]  # only used by log handler
+    sync_branch_id: Optional[int] = None  # only used by log handler
     connection_id: int  # only used by log handler
     run_id: int  # used by log handler and for reporting back run status updates
     source_app_name: str  # the name of the app which is invoking this plugin
@@ -145,14 +147,15 @@ class InboundSyncRequestPayload(BaseModel):
     logging_level: str
     connection_method: str
     connection_parameters: Dict[str, StoredConfigurationValue]
-    oauth_secret_name: Optional[str]
-    other_secrets_name: Optional[str]
+    oauth_secret_name: Optional[str] = None
+    other_secrets_name: Optional[str] = None
     sync_direction: Literal["inbound"] = "inbound"
     sync_parameters: Dict[str, StoredConfigurationValue]
     api_limit_overrides: List[ApiLimits]
     rate_limits_state: Dict[int,Dict[str,Dict[str,RateLimitState]]]
     streams_configuration: InboundSyncStreamsConfiguration
     latest_stream_state: Dict
+    time_limit_mins: int = 60 * 4
 
 
 SyncRequestPayload = Annotated[
