@@ -3,12 +3,14 @@ Omnata Plugin Runtime configuration objects.
 Includes data container classes related to plugin configuration.
 """
 from __future__ import annotations
+import json
 import sys
+import logging
 from typing import Any, List, Dict, Literal, Union, Optional
 from enum import Enum
 
 from abc import ABC
-from pydantic import BaseModel, Field, validator  # pylint: disable=no-name-in-module
+from pydantic import BaseModel, Field, PrivateAttr, SerializationInfo, TypeAdapter, field_validator, model_serializer, validator  # pylint: disable=no-name-in-module
 
 if tuple(sys.version_info[:2]) >= (3, 9):
     # Python 3.9 and above
@@ -17,6 +19,8 @@ else:
     # Python 3.8 and below
     from typing_extensions import Annotated
 
+logger = logging.getLogger(__name__)
+
 
 class MapperType(str, Enum):
     FIELD_MAPPING_SELECTOR = "field_mapping_selector"
@@ -31,12 +35,13 @@ class SyncDirection(str, Enum):
 class InboundStorageBehaviour(str, Enum):
     APPEND = "append"
     MERGE = "merge"
-    REPLACE = "replace"
+    REPLACE = "replace" # now deprecated, full refreshes now use merge
 
 
 class InboundSyncStrategy(str, Enum):
     FULL_REFRESH = "Full Refresh"
     INCREMENTAL = "Incremental"
+    AUTO = "Auto" # not valid as an individual stream choice, only when referring to a forward preference (new_stream_sync_strategy)
 
 
 class InboundSyncBulkConfiguration(str, Enum):
@@ -44,11 +49,14 @@ class InboundSyncBulkConfiguration(str, Enum):
     Provides a way to apply a combination of sync strategy and storage behaviour to all selected streams.
     These options will be reduce to the set of supported strategies. If there is no overlap, only CUSTOMIZE will be supported
     """
+    AUTO_MERGE = "Auto / Merge changes" # Merge
+    AUTO_APPEND = "Auto / Keep history"  # Append
+    CUSTOMIZE = "Customize - choose sync behaviours for each object"
+    # These are legacy values, back when we used to let people bulk set the full combination
     ALL_FULL_REPLACE = "All Objects - Full refresh / Replace"
     ALL_FULL_APPEND = "All Objects - Full refresh / Append"
     ALL_INCR_MERGE = "All Objects - Incremental / Merge"
     ALL_INCR_APPEND = "All Objects - Incremental / Append"
-    CUSTOMIZE = "Customize - choose sync behaviours for each object"
 
 
 ICON_URL_CODE = '<svg xmlns="http://www.w3.org/2000/svg" width="100%" height="100%" viewBox="0 0 24 24"><path fill="currentColor" d="m8 18l-6-6l6-6l1.425 1.425l-4.6 4.6L9.4 16.6Zm8 0l-1.425-1.425l4.6-4.6L14.6 7.4L16 6l6 6Z"/></svg>'
@@ -106,16 +114,19 @@ class OutboundSyncAction(SubscriptableBaseModel, ABC):
                     **STANDARD_OUTBOUND_SYNC_ACTIONS[data["action_name"]]().__dict__,
                 }
         super().__init__(**data)
-
-    def dict(self, *args, trim: bool = True, **kwargs) -> Dict[str, any]:
-        """
-        OutboundSyncStrategy objects can be serialized much smaller if the instance is of a Standard strategy that resides in the
-        Omnata plugin runtime. This method trims out the unneeded attributes in these scenarios.
-        This method also trims the actions for this strategy if it's a custom one.
-
-        """
-        excluded_fields = {} if not trim or self.custom_action else {"description"}
-        return super().dict(exclude=excluded_fields)
+    
+    @model_serializer(mode='wrap')
+    # no return type hint, see https://github.com/fastapi/fastapi/discussions/10661#discussioncomment-7631104
+    def ser_model(self,handler,info:SerializationInfo):
+        serialized:Dict[str,Any] = handler(self)
+        if not self.custom_action and (info.exclude_none is None or info.exclude_none == False):
+            return {k:v for k,v in serialized.items() if k not in [
+                "description"]}
+        return serialized
+    
+    def model_dump_no_trim(self) -> Dict[str, Any]:
+        # we use our own special include value to signal not to trim
+        return self.model_dump(exclude_none=True)
 
 
 class CreateSyncAction(OutboundSyncAction):
@@ -217,13 +228,13 @@ class OutboundSyncStrategy(SubscriptableBaseModel, ABC):
     name: str
     description: str
     icon_source: str = ICON_URL_CODE
-    action_on_record_create: Optional[OutboundSyncAction] = None
-    action_on_record_update: Optional[OutboundSyncAction] = None
-    action_on_record_delete: Optional[OutboundSyncAction] = None
-    action_on_record_unchanged: Optional[OutboundSyncAction] = None
+    action_on_record_create: Optional[OutboundSyncAction] = Field(default=None)
+    action_on_record_update: Optional[OutboundSyncAction] = Field(default=None)
+    action_on_record_delete: Optional[OutboundSyncAction] = Field(default=None)
+    action_on_record_unchanged: Optional[OutboundSyncAction] = Field(default=None)
     custom_strategy: bool = True
 
-    def __eq__(self, other):
+    def __eq__(self, other:OutboundSyncStrategy):
         if hasattr(other, 'custom_strategy') and hasattr(other, 'name'):
             return (
                 self.custom_strategy == other.custom_strategy
@@ -247,27 +258,24 @@ class OutboundSyncStrategy(SubscriptableBaseModel, ABC):
                 }
         super().__init__(**data)
 
-    def dict(self, *args, trim: bool = True, **kwargs) -> Dict[str, any]:
-        """
-        OutboundSyncStrategy objects can be serialized much smaller if the instance is of a Standard strategy that resides in the
-        Omnata plugin runtime. This method trims out the unneeded attributes in these scenarios.
-        This method also trims the actions for this strategy if it's a custom one.
-
-        """
-        excluded_fields = (
-            {}
-            if not trim or self.custom_strategy
-            else {
+    @model_serializer(mode='wrap')
+    # no return type hint, see https://github.com/fastapi/fastapi/discussions/10661#discussioncomment-7631104
+    def ser_model(self,handler,info:SerializationInfo):
+        serialized:Dict[str,Any] = handler(self)
+        if not self.custom_strategy and (info.exclude_none is None or info.exclude_none == False):
+            return {k:v for k,v in serialized.items() if k not in [
                 "description",
                 "icon_source",
                 "action_on_record_create",
                 "action_on_record_update",
                 "action_on_record_delete",
-                "action_on_record_unchanged",
-            }
-        )
-        return super().dict(exclude=excluded_fields)
-
+                "action_on_record_unchanged"]}
+        return serialized
+    
+    def model_dump_no_trim(self) -> Dict[str, Any]:
+        # we use our own special include value to signal not to trim
+        return self.model_dump(exclude_none=True)
+    
 
 class CreateSyncStrategy(OutboundSyncStrategy):
     """
@@ -420,8 +428,8 @@ class InboundSyncStreamsConfiguration(SubscriptableBaseModel):
     """
 
     include_new_streams: bool
-    new_stream_sync_strategy: Optional[InboundSyncStrategy]
-    new_stream_storage_behaviour: Optional[InboundStorageBehaviour]
+    new_stream_sync_strategy: Optional[InboundSyncStrategy] = Field(default=None)
+    new_stream_storage_behaviour: Optional[InboundStorageBehaviour] = Field(default=None)
     included_streams: Dict[str, StoredStreamConfiguration]
     excluded_streams: List[str]
     bulk_configuration: InboundSyncBulkConfiguration = InboundSyncBulkConfiguration.CUSTOMIZE
@@ -439,15 +447,21 @@ class StoredStreamConfiguration(SubscriptableBaseModel):
         None,
         description="The field to use as a cursor",
     )
-    # Composite primary keys is not really a thing in SaaS applications
-    primary_key_field: Optional[str] = Field(
+    primary_key_field: Optional[Union[str,List[str]]] = Field(
         None,
-        description="The field that will be used as primary key.",
+        description="The field(s) that will be used as primary key.",
     )
     storage_behaviour: InboundStorageBehaviour
     stream: StreamConfiguration
     latest_state: dict = Field(default_factory=dict,description="The latest state of the stream, used for incremental syncs")
 
+    @field_validator('latest_state',mode='before')
+    @classmethod
+    def state_must_not_be_none(cls, v: Optional[dict]) -> dict:
+        if v is None:
+            return {}
+        return v
+
 
 class StreamConfiguration(SubscriptableBaseModel):
     """
@@ -461,9 +475,17 @@ class StreamConfiguration(SubscriptableBaseModel):
         None,
         description="If true, the plugin controls the cursor field",
     )
-    source_defined_primary_key: Optional[str] = Field(
+    source_defined_primary_key: Optional[Union[str,List[str]]] = Field(
         None,
-        description="If true, the plugin controls the primary key",
+        description="If defined, the plugin controls the primary key field",
+    )
+    # For SaaS applications, typically the primary key is a single field, known to the plugin.
+    # For databases, the primary key can be composite, and the plugin may not know what it is if there is no key defined explicitly on the table.
+    # this flag was added to support this scenario. If it is to True, then source_defined_primary_key will be a list of fields.
+    # When this occurs, the plugin runtime will build the APP_IDENTIFIER from the fields in the order they are listed, by concatenating them.
+    primary_key_can_be_composite: Optional[bool] = Field(
+        False,
+        description="If true, the primary key can be composite",
     )
     default_cursor_field: Optional[str] = Field(
         None,
@@ -521,14 +543,14 @@ class ConnectionConfigurationParameters(SubscriptableBaseModel):
     """
 
     connection_method: str
-    connection_parameters: Dict[str, StoredConfigurationValue] = None
-    connection_secrets: Dict[str, StoredConfigurationValue] = None
-    ngrok_tunnel_settings: Optional[NgrokTunnelSettings] = None
-    access_token_secret_name: Optional[str] = None
-    _snowflake: Optional[Any] = None
+    connection_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    connection_secrets: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    ngrok_tunnel_settings: Optional[NgrokTunnelSettings] = Field(default=None)
+    access_token_secret_name: Optional[str] = Field(default=None)
 
-    class Config:
-        underscore_attrs_are_private = True
+    _snowflake: Optional[Any] = PrivateAttr(  # or use Any to annotate the type and use Field to initialize
+        default=None
+    )
 
     @validator("ngrok_tunnel_settings", always=True)
     def validate_ngrok_tunnel_settings(cls, v: str, values: dict[str, Any]) -> Optional[NgrokTunnelSettings]:
@@ -597,10 +619,10 @@ class SyncConfigurationParameters(ConnectionConfigurationParameters):
     """
 
     connection_method: str
-    connection_parameters: Dict[str, StoredConfigurationValue] = None
-    connection_secrets: Dict[str, StoredConfigurationValue] = None
-    sync_parameters: Dict[str, StoredConfigurationValue] = None
-    current_form_parameters: Dict[str, StoredConfigurationValue] = None
+    connection_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    connection_secrets: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    sync_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    current_form_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
 
     def sync_parameter_exists(self, parameter_name: str) -> bool:
         """
@@ -677,12 +699,12 @@ class OutboundSyncConfigurationParameters(SyncConfigurationParameters):
     """
 
     connection_method: str
-    connection_parameters: Dict[str, StoredConfigurationValue] = None
-    connection_secrets: Dict[str, StoredConfigurationValue] = None
-    sync_parameters: Dict[str, StoredConfigurationValue] = None
-    current_form_parameters: Dict[str, StoredConfigurationValue] = None
-    sync_strategy: OutboundSyncStrategy = None
-    field_mappings: StoredMappingValue = None
+    connection_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    connection_secrets: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    sync_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    current_form_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    sync_strategy: Optional[OutboundSyncStrategy] = Field(default=None)
+    field_mappings: Optional[StoredMappingValue] = Field(default=None)
 
 
 class InboundSyncConfigurationParameters(SyncConfigurationParameters):
@@ -697,11 +719,11 @@ class InboundSyncConfigurationParameters(SyncConfigurationParameters):
     """
 
     connection_method: str
-    connection_parameters: Dict[str, StoredConfigurationValue] = None
-    connection_secrets: Dict[str, StoredConfigurationValue] = None
-    sync_parameters: Dict[str, StoredConfigurationValue] = None
-    current_form_parameters: Dict[str, StoredConfigurationValue] = None
-    currently_selected_streams: List[str] = None
+    connection_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    connection_secrets: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    sync_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    current_form_parameters: Optional[Dict[str, StoredConfigurationValue]] = Field(default=None)
+    currently_selected_streams: Optional[List[str]] = Field(default=None)
 
 
 class StoredJinjaTemplate(SubscriptableBaseModel):
@@ -711,7 +733,7 @@ class StoredJinjaTemplate(SubscriptableBaseModel):
     """
 
     mapper_type: Literal["jinja_template"] = "jinja_template"
-    additional_column_expressions: Dict[str, str] = {}
+    additional_column_expressions: Dict[str, str] = Field(default={})
     jinja_template: str
 
 
@@ -745,14 +767,14 @@ class SyncScheduleSnowflakeTask(SubscriptableBaseModel):
 
     """
 
-    mode: Literal["snowflake_task"]
+    mode: Literal["snowflake_task"] = "snowflake_task"
     sync_frequency: str
     sync_frequency_name: Literal["1 min", "5 mins", "15 mins", "Hourly", "Daily","Custom"]
-    warehouse: str = None
+    warehouse: Optional[str] = Field(default=None)
     time_limit_mins: int = 60 * 4
-    daily_hour: Optional[int] = None
-    daily_minute: Optional[int] = None
-    minute_of_hour: Optional[int] = None
+    daily_hour: Optional[int] = Field(default=None)
+    daily_minute: Optional[int] = Field(default=None)
+    minute_of_hour: Optional[int] = Field(default=None)
 
 
 class SyncScheduleDbt(SubscriptableBaseModel):
@@ -770,7 +792,7 @@ class SyncScheduleDbt(SubscriptableBaseModel):
         is_dbt_cloud (bool): If true, dbt cloud is in use
     """
 
-    mode: Literal["dbt"]
+    mode: Literal["dbt"] = "dbt"
     dbt_prod_target_name: str = "prod"
     task_warehouse_dbt_defined: bool = True
     warehouse: str
@@ -792,7 +814,7 @@ class SyncScheduleDependant(SubscriptableBaseModel):
         selected_sync (int): The sync ID of the sync to depend on
     """
 
-    mode: Literal["dependent"]
+    mode: Literal["dependent"] = "dependent"
     run_when: Literal[
         "after_parent_completes", "at_same_time_as"
     ] = "after_parent_completes"
@@ -810,7 +832,7 @@ class SyncScheduleManual(SubscriptableBaseModel):
         time_limit_mins (int): The maximum time the sync can run for before being cancelled
     """
 
-    mode: Literal["manual"]
+    mode: Literal["manual"] = "manual"
     warehouse: str
     time_limit_mins: int = 60 * 4
 
@@ -838,7 +860,33 @@ class StoredFieldMapping(SubscriptableBaseModel):
     app_metadata: dict = Field(default_factory=dict)
 
 
-StoredStreamConfiguration.update_forward_refs()
-InboundSyncStreamsConfiguration.update_forward_refs()
-StoredFieldMappings.update_forward_refs()
-OutboundSyncConfigurationParameters.update_forward_refs()
+StoredStreamConfiguration.model_rebuild()
+InboundSyncStreamsConfiguration.model_rebuild()
+StoredFieldMappings.model_rebuild()
+OutboundSyncConfigurationParameters.model_rebuild()
+
+
+def get_secrets(oauth_secret_name: Optional[str], other_secrets_name: Optional[str]
+) -> Dict[str, StoredConfigurationValue]:
+    connection_secrets = {}
+    import _snowflake # pylint: disable=import-error, import-outside-toplevel # type: ignore
+    if oauth_secret_name is not None:
+        connection_secrets["access_token"] = StoredConfigurationValue(
+            value=_snowflake.get_oauth_access_token(oauth_secret_name)
+        )
+    if other_secrets_name is not None:
+        try:
+            secret_string_content = _snowflake.get_generic_secret_string(
+                other_secrets_name
+            )
+            if len(secret_string_content) > 2:
+                other_secrets = json.loads(secret_string_content)
+                connection_secrets = {
+                    **connection_secrets,
+                    **TypeAdapter(Dict[str, StoredConfigurationValue]).validate_python(other_secrets),
+                }
+        except Exception as exception:
+            logger.error(f"Error parsing secrets content for secret {other_secrets_name}: {str(exception)}")
+            raise ValueError(f"Error parsing secrets content: {str(exception)}") from exception
+    return connection_secrets
+    
\ No newline at end of file
