@@ -41,10 +41,10 @@ class FormOption(SubscriptableBaseModel):
     value: str
     label: str
     metadata: dict = Field(default_factory=dict)
-    required: bool = False
-    unique: bool = False
-    default: bool = False
-    disabled: bool = False
+    required: bool = Field(default=False)
+    unique: bool = Field(default=False)
+    default: bool = Field(default=False)
+    disabled: bool = Field(default=False)
     data_type_icon: str = "unknown"
 
 
@@ -55,12 +55,12 @@ class FormInputField(SubscriptableBaseModel):
 
     name: str
     label: str
-    default_value: Union[str, bool] = ""
-    required: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
-    reload_on_change: bool = False
-    secret: bool = False
+    default_value: Union[str, bool] = Field(default="")
+    required: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
+    reload_on_change: bool = Field(default=False)
+    secret: bool = Field(default=False)
     type: Literal["input"] = "input"
 
 
@@ -81,15 +81,15 @@ class FormTextAreaField(SubscriptableBaseModel):
 
     name: str
     label: str
-    default_value: str = ""
-    secret: bool = False
-    required: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
-    reload_on_change: bool = False
+    default_value: str = Field(default="")
+    secret: bool = Field(default=False)
+    required: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
+    reload_on_change: bool = Field(default=False)
     type: Literal["textarea"] = "textarea"
 
-    variables: bool = False
+    variables: bool = Field(default=False)
 
 class InformationField(SubscriptableBaseModel):
     """
@@ -101,9 +101,10 @@ class InformationField(SubscriptableBaseModel):
     """
     name: str
     markdown_content: str
-    depends_on: Optional[str] = None
+    depends_on: Optional[str] = Field(default=None)
     type: Literal["information"] = "information"
-    reload_on_change: bool = False
+    reload_on_change: bool = Field(default=False)
+    secret: bool = Field(default=False)
 
 class InformationBoxField(SubscriptableBaseModel):
     """
@@ -118,11 +119,12 @@ class InformationBoxField(SubscriptableBaseModel):
     """
     name: str
     markdown_content: str
-    depends_on: Optional[str] = None
+    depends_on: Optional[str] = Field(default=None)
     type: Literal["information_box"] = "information_box"
-    reload_on_change: bool = False
+    reload_on_change: bool = Field(default=False)
     box_type: Literal["info", "warning", "error"] = "info"
-    box_icon: Optional[str] = None
+    box_icon: Optional[str] = Field(default=None)
+    secret: bool = Field(default=False)
 
 class FormSshKeypair(SubscriptableBaseModel):
     """
@@ -130,21 +132,21 @@ class FormSshKeypair(SubscriptableBaseModel):
     """
     name: str
     label: str
-    default_value: Optional[str] = None
-    required: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
+    default_value: Optional[str] = Field(default=None)
+    required: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
     local_side: Literal["public", "private"] = "public"
     """
     The side of the keypair which belongs to Snowflake. This value will be stored as the secret value, and the other side
     can be copied/downloaded only during the connection process.
     """
-    allow_user_provided: bool = True
+    allow_user_provided: bool = Field(default=True)
     """
     Allows the user to provide Snowflake's side of the keypair, for cases where one is generated on the outside
     """
     type: Literal["ssh_keypair"] = "ssh_keypair"
-    secret: bool = True
+    secret: bool = Field(default=True)
 
 class FormX509Certificate(SubscriptableBaseModel):
     """
@@ -153,12 +155,12 @@ class FormX509Certificate(SubscriptableBaseModel):
     """
     name: str
     label: str
-    default_value: Optional[str] = None
-    required: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
+    default_value: Optional[str] = Field(default=None)
+    required: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
     type: Literal["x509_certificate"] = "x509_certificate"
-    secret: bool = True
+    secret: bool = Field(default=True)
 
 class FormGpgKeypair(SubscriptableBaseModel):
     """
@@ -167,21 +169,21 @@ class FormGpgKeypair(SubscriptableBaseModel):
 
     name: str
     label: str
-    default_value: Optional[str] = None
-    required: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
+    default_value: Optional[str] = Field(default=None)
+    required: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
     local_side: Literal["public", "private"] = "public"
     """
     The side of the keypair which belongs to Snowflake. This value will be stored as the secret value, and the other side
     can be copied/downloaded only during the connection process.
     """
-    allow_user_provided: bool = True
+    allow_user_provided: bool = Field(default=True)
     """
     Allows the user to provide Snowflake's side of the keypair, for cases where one is generated on the outside
     """
     type: Literal["gpg_keypair"] = "gpg_keypair"
-    secret: bool = True
+    secret: bool = Field(default=True)
 
 
 class FormCheckboxField(SubscriptableBaseModel):
@@ -191,12 +193,12 @@ class FormCheckboxField(SubscriptableBaseModel):
 
     name: str
     label: str
-    default_value: bool = False
-    required: bool = False
-    secret: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
-    reload_on_change: bool = False
+    default_value: bool = Field(default=False)
+    required: bool = Field(default=False)
+    secret: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
+    reload_on_change: bool = Field(default=False)
     type: Literal["checkbox"] = "checkbox"
 
 
@@ -207,17 +209,17 @@ class FormSliderField(SubscriptableBaseModel):
 
     name: str
     label: str
-    default_value: Optional[str] = None
-    secret: bool = False
-    required: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
-    reload_on_change: bool = False
+    default_value: Optional[Union[str,int]] = Field(default=None)
+    secret: bool = Field(default=False)
+    required: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
+    reload_on_change: bool = Field(default=False)
     type: Literal["slider"] = "slider"
 
-    min_value: int = 0
-    max_value: int = 100
-    step_size: int = 1
+    min_value: int = Field(default=0)
+    max_value: int = Field(default=100)
+    step_size: int = Field(default=1)
 
 
 class FormJinjaTemplate(SubscriptableBaseModel):
@@ -227,7 +229,7 @@ class FormJinjaTemplate(SubscriptableBaseModel):
 
     mapper_type: Literal["jinja_template"] = "jinja_template"
     label: str = "Jinja Template"
-    depends_on: Optional[str] = None
+    depends_on: Optional[str] = Field(default=None)
 
 
 # ----------------------------------------------------------------------------
@@ -247,7 +249,7 @@ class StaticFormOptionsDataSource(SubscriptableBaseModel):
     """
 
     values: List[FormOption] = Field(default_factory=list)
-    new_option_creator: Optional[NewOptionCreator] = None
+    new_option_creator: Optional[NewOptionCreator] = Field(default=None)
     type: Literal["static"] = "static"
 
 
@@ -259,7 +261,7 @@ class DynamicFormOptionsDataSource(SubscriptableBaseModel):
     source_function: Union[
         Callable[[SyncConfigurationParameters], List[FormOption]], str
     ]
-    new_option_creator: Optional[NewOptionCreator] = None
+    new_option_creator: Optional[NewOptionCreator] = Field(default=None)
     type: Literal["dynamic"] = "dynamic"
 
     @validator("source_function", always=True)
@@ -297,12 +299,12 @@ class FormRadioField(FormFieldWithDataSource, BaseModel):
 
     name: str
     label: str
-    default_value: Optional[str] = None
-    required: bool = False
-    secret: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
-    reload_on_change: bool = False
+    default_value: Optional[str] = Field(default=None)
+    required: bool = Field(default=False)
+    secret: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
+    reload_on_change: bool = Field(default=False)
     type: Literal["radio"] = "radio"
 
 
@@ -314,15 +316,15 @@ class FormDropdownField(FormFieldWithDataSource, BaseModel):
 
     name: str
     label: str
-    default_value: Optional[str] = None
-    required: bool = False
-    secret: bool = False
-    depends_on: Optional[str] = None
-    help_text: str = ""
-    reload_on_change: bool = False
+    default_value: Optional[str] = Field(default=None)
+    required: bool = Field(default=False)
+    secret: bool = Field(default=False)
+    depends_on: Optional[str] = Field(default=None)
+    help_text: str = Field(default="")
+    reload_on_change: bool = Field(default=False)
     type: Literal["dropdown"] = "dropdown"
 
-    multi_select: bool = False
+    multi_select: bool = Field(default=False)
 
 
 FormFieldBase = Annotated[
@@ -372,7 +374,7 @@ class NewOptionCreator(SubscriptableBaseModel):
     construct_form_option: Union[
         Callable[[StoredConfigurationValue], FormOption], str
     ]
-    allow_create: bool = True
+    allow_create: bool = Field(default=True)
 
     @validator("creation_form_function", always=True)
     def function_name_convertor(cls, v) -> str:
@@ -387,8 +389,8 @@ class NewOptionCreator(SubscriptableBaseModel):
         return v.__name__ if isinstance(v, MethodType) else v
 
 
-StaticFormOptionsDataSource.update_forward_refs()
-DynamicFormOptionsDataSource.update_forward_refs()
+StaticFormOptionsDataSource.model_rebuild()
+DynamicFormOptionsDataSource.model_rebuild()
 
 
 class FormFieldMappingSelector(FormFieldWithDataSource, BaseModel):
@@ -402,7 +404,7 @@ class FormFieldMappingSelector(FormFieldWithDataSource, BaseModel):
 
     mapper_type: Literal["field_mapping_selector"] = "field_mapping_selector"
     label: str = "Field Mappings"
-    depends_on: Optional[str] = None
+    depends_on: Optional[str] = Field(default=None)
 
 
 Mapper = Annotated[
@@ -418,7 +420,7 @@ class OutboundSyncConfigurationForm(ConfigurationFormBase):
     to map Snowflake columns to app fields/payloads.
     """
 
-    mapper: Optional[Mapper] = None
+    mapper: Optional[Mapper] = Field(default=None)
 
 
 class InboundSyncConfigurationForm(ConfigurationFormBase):
@@ -434,24 +436,24 @@ class SecurityIntegrationTemplateAuthorizationCode(BaseModel):
     Provides values used to populate a security integration instructions template, which
     in turn allows the customer to create an OAuth based secret object
     """
-    oauth_docs_url: Optional[str] = None
+    oauth_docs_url: Optional[str] = Field(default=None)
     oauth_grant: Literal["authorization_code"] = "authorization_code"
-    oauth_client_id: str = '<client id>'
-    oauth_client_secret: str = '<client secret>'
-    oauth_token_endpoint: str = '<token endpoint>'
-    oauth_authorization_endpoint: str = '<authorization endpoint>'
-    oauth_allowed_scopes: List[str] = []
+    oauth_client_id: str = Field(default='<client id>')
+    oauth_client_secret: str = Field(default='<client secret>')
+    oauth_token_endpoint: str = Field(default='<token endpoint>')
+    oauth_authorization_endpoint: str = Field(default='<authorization endpoint>')
+    oauth_allowed_scopes: List[str] = Field(default=[])
 
 class SecurityIntegrationTemplateClientCredentials(BaseModel):
     """
     Provides values used to populate a security integration instructions template, which
     in turn allows the customer to create an OAuth based secret object
     """
-    oauth_docs_url: Optional[str] = None
+    oauth_docs_url: Optional[str] = Field(default=None)
     oauth_grant: Literal["client_credentials"] = "client_credentials"
-    oauth_client_id: str = '<client id>'
-    oauth_client_secret: str = '<client secret>'
-    oauth_token_endpoint: str = '<token endpoint>'
+    oauth_client_id: str = Field(default='<client id>')
+    oauth_client_secret: str = Field(default='<client secret>')
+    oauth_token_endpoint: str = Field(default='<token endpoint>')
     oauth_allowed_scopes: List[str] = []
 
 SecurityIntegrationTemplate = Annotated[
@@ -463,7 +465,7 @@ class NGrokMTLSTunnel(SubscriptableBaseModel):
     """
     Designates a ConnectionMethod as connecting via an ngrok tunnel.
     """
-    SupportEdgeTermination: bool = True
+    SupportEdgeTermination: bool = Field(default=True)
     post_tunnel_fields_function: Union[
         Callable[[ConnectionConfigurationParameters], List[FormFieldBase]], str
     ]
@@ -486,6 +488,6 @@ class ConnectionMethod(SubscriptableBaseModel):
 
     name: str
     fields: List[FormFieldBase]
-    oauth_template: Optional[SecurityIntegrationTemplate] = None
-    ngrok_tunnel_configuration: Optional[NGrokMTLSTunnel] = None
-    description: str = ""
+    oauth_template: Optional[SecurityIntegrationTemplate] = Field(default=None)
+    ngrok_tunnel_configuration: Optional[NGrokMTLSTunnel] = Field(default=None)
+    description: str = Field(default="")
