@@ -8,11 +8,11 @@ import time
 import threading
 from typing import Dict, List, Optional
 
-from pydantic import BaseModel, parse_obj_as  # pylint: disable=no-name-in-module
-import pydantic.json
+from pydantic import BaseModel,TypeAdapter  # pylint: disable=no-name-in-module
+from pydantic_core import to_jsonable_python
 from snowflake.snowpark import Session
 
-from .api import SyncRequestPayload, handle_proc_result
+from .api import PluginMessageStreamProgressUpdate, SyncRequestPayload, handle_proc_result
 from .configuration import (
     ConnectionConfigurationParameters,
     InboundSyncConfigurationParameters,
@@ -20,6 +20,7 @@ from .configuration import (
     OutboundSyncStrategy,
     StoredConfigurationValue,
     StoredMappingValue,
+    get_secrets
 )
 from .forms import ConnectionMethod, FormInputField, FormOption
 from .logging import OmnataPluginLogHandler
@@ -32,6 +33,7 @@ from .omnata_plugin import (
     OutboundSyncRequest,
     DeadlineReachedException,
 )
+from pydantic import TypeAdapter
 from .rate_limiting import ApiLimits, RateLimitState
 
 # set the logger class to our custom logger so that pydantic errors are handled correctly
@@ -57,11 +59,30 @@ class PluginEntrypoint:
         class_obj = getattr(module, class_name)
         self._plugin_instance: OmnataPlugin = class_obj()
         self._plugin_instance._session = session  # pylint: disable=protected-access
+        # logging defaults
+        snowflake_logger = logging.getLogger("snowflake")
+        snowflake_logger.setLevel(logging.WARN) # we don't want snowflake queries being logged by default
+        # the sync engine can tell the plugin to override log level via a session variable
+        if session is not None:
+            try:
+                v = session.sql("select getvariable('LOG_LEVEL_OVERRIDES')").collect()
+                result = v[0][0]
+                if result is not None:
+                    log_level_overrides:Dict[str,str] = json.loads(result)
+                    for logger_name,level in log_level_overrides.items():
+                        logger_override = logging.getLogger(logger_name)
+                        logger_override.setLevel(level)
+                        logger_override.propagate = False
+                        for handler in logger_override.handlers:
+                            handler.setLevel(level)
+            except Exception as e:
+                logger.error(f"Error setting log level overrides: {str(e)}")
+        
 
     def sync(self, sync_request: Dict):
         logger.info("Entered sync method")
-        request = parse_obj_as(SyncRequestPayload, sync_request)
-        connection_secrets = self.get_secrets(
+        request = TypeAdapter(SyncRequestPayload).validate_python(sync_request)
+        connection_secrets = get_secrets(
             request.oauth_secret_name, request.other_secrets_name
         )
         omnata_log_handler = OmnataPluginLogHandler(
@@ -84,7 +105,7 @@ class PluginEntrypoint:
             connection_parameters.access_token_secret_name = request.oauth_secret_name
         all_api_limits = self._plugin_instance.api_limits(connection_parameters)
         logger.info(
-            f"Default API limits: {json.dumps(all_api_limits, default=pydantic.json.pydantic_encoder)}"
+            f"Default API limits: {json.dumps(to_jsonable_python(all_api_limits))}"
         )
         all_api_limits_by_category = {
             api_limit.endpoint_category: api_limit for api_limit in all_api_limits
@@ -100,7 +121,7 @@ class PluginEntrypoint:
         api_limits = list(all_api_limits_by_category.values())
         return_dict = {}
         logger.info(
-            f"Rate limits state: {json.dumps(request.rate_limits_state, default=pydantic.json.pydantic_encoder)}"
+            f"Rate limits state: {json.dumps(to_jsonable_python(request.rate_limits_state))}"
         )
         (rate_limit_state_all, rate_limit_state_this_branch) = RateLimitState.collapse(request.rate_limits_state,request.sync_id, request.sync_branch_name)
         # if any endpoint categories have no state, give them an empty state
@@ -209,7 +230,23 @@ class PluginEntrypoint:
                     inbound_sync_request.update_activity("Staging remaining records")
                     logger.info("Calling apply_results_queue")
                     inbound_sync_request.apply_results_queue()
-                    inbound_sync_request.apply_rate_limit_state()
+                    try:
+                        # this is not critical, we wouldn't fail the sync over rate limit usage capture
+                        logger.info("Calling apply_rate_limit_state")
+                        inbound_sync_request.apply_rate_limit_state()
+                    except Exception as e:
+                        logger.error(f"Error applying rate limit state: {str(e)}")
+                    # here we used to do a final inbound_sync_request.apply_progress_updates(ignore_errors=False)
+                    # but it was erroring too much since there was usually a lot of DDL activity on the Snowflake side
+                    # so instead, we'll provide a final progress update via a return value from the proc
+                final_progress_update = PluginMessageStreamProgressUpdate(
+                    stream_total_counts=inbound_sync_request._stream_record_counts,
+                    completed_streams=inbound_sync_request._completed_streams,
+                    stream_errors=omnata_log_handler.stream_global_errors,
+                    total_records_estimate=inbound_sync_request._total_records_estimate
+                )
+                return_dict["streams_requiring_view_refresh"] = inbound_sync_request.streams_requiring_view_refresh
+                return_dict["final_progress_update"] = final_progress_update.model_dump()
                 if inbound_sync_request.deadline_reached:
                     # if we actually hit the deadline, this is flagged by the cancellation checking worker and the cancellation
                     # token is set. We throw it here as an error since that's currently how it flows back to the engine with a DELAYED state
@@ -226,11 +263,6 @@ class PluginEntrypoint:
                         inbound_sync_request._rate_limit_update_task.join()  # pylint: disable=protected-access
                 except Exception as e:
                     logger.error(f"Error cleaning up threading: {str(e)}")
-            return_dict["errored_streams"] = list(
-                omnata_log_handler.stream_global_errors.keys()
-            )
-            # we need to calculate counts for:
-            # CHANGED_COUNT by counting up the records in INBOUND_STREAM_RECORD_COUNTS
         logger.info("Finished applying records")
         return return_dict
 
@@ -250,20 +282,16 @@ class PluginEntrypoint:
         sync_strategy = normalise_nulls(sync_strategy)
         oauth_secret_name = normalise_nulls(oauth_secret_name)
         other_secrets_name = normalise_nulls(other_secrets_name)
-        connection_secrets = self.get_secrets(oauth_secret_name, other_secrets_name)
-        connection_parameters = parse_obj_as(
-            Dict[str, StoredConfigurationValue], connection_parameters
-        )
-        sync_parameters = parse_obj_as(
-            Dict[str, StoredConfigurationValue], sync_parameters
-        )
+        connection_secrets = get_secrets(oauth_secret_name, other_secrets_name)
+        connection_parameters = TypeAdapter(
+            Dict[str, StoredConfigurationValue]).validate_python(connection_parameters)
+        sync_parameters = TypeAdapter(
+            Dict[str, StoredConfigurationValue]).validate_python(sync_parameters)
         form_parameters = None
         if current_form_parameters is not None:
-            form_parameters = parse_obj_as(
-                Dict[str, StoredConfigurationValue], current_form_parameters
-            )
+            form_parameters = TypeAdapter(Dict[str, StoredConfigurationValue]).validate_python(current_form_parameters)
         if sync_direction == "outbound":
-            sync_strat = OutboundSyncStrategy.parse_obj(sync_strategy) if sync_strategy is not None else None
+            sync_strat = OutboundSyncStrategy.model_validate(sync_strategy) if sync_strategy is not None else None
             parameters = OutboundSyncConfigurationParameters(
                 connection_parameters=connection_parameters,
                 connection_secrets=connection_secrets,
@@ -290,10 +318,10 @@ class PluginEntrypoint:
         )
         script_result = the_function(parameters)
         if isinstance(script_result, BaseModel):
-            script_result = script_result.dict()
+            script_result = script_result.model_dump()
         elif isinstance(script_result, List):
             if len(script_result) > 0 and isinstance(script_result[0], BaseModel):
-                script_result = [r.dict() for r in script_result]
+                script_result = [r.model_dump() for r in script_result]
         return script_result
         
     def inbound_list_streams(
@@ -305,16 +333,14 @@ class PluginEntrypoint:
         sync_parameters: Dict,
         selected_streams: Optional[List[str]], # None to return all streams without requiring schema
     ):
-        logger.info("Entered list_streams method")
+        logger.debug("Entered list_streams method")
         oauth_secret_name = normalise_nulls(oauth_secret_name)
         other_secrets_name = normalise_nulls(other_secrets_name)
-        connection_secrets = self.get_secrets(oauth_secret_name, other_secrets_name)
-        connection_parameters = parse_obj_as(
-            Dict[str, StoredConfigurationValue], connection_parameters
-        )
-        sync_parameters = parse_obj_as(
-            Dict[str, StoredConfigurationValue], sync_parameters
-        )
+        connection_secrets = get_secrets(oauth_secret_name, other_secrets_name)
+        connection_parameters = TypeAdapter(
+            Dict[str, StoredConfigurationValue]).validate_python(connection_parameters)
+        sync_parameters = TypeAdapter(
+            Dict[str, StoredConfigurationValue]).validate_python(sync_parameters)
         parameters = InboundSyncConfigurationParameters(
             connection_parameters=connection_parameters,
             connection_secrets=connection_secrets,
@@ -328,10 +354,10 @@ class PluginEntrypoint:
 
         script_result = self._plugin_instance.inbound_stream_list(parameters)
         if isinstance(script_result, BaseModel):
-            script_result = script_result.dict()
+            script_result = script_result.model_dump()
         elif isinstance(script_result, List):
             if len(script_result) > 0 and isinstance(script_result[0], BaseModel):
-                script_result = [r.dict() for r in script_result]
+                script_result = [r.model_dump() for r in script_result]
         return script_result
 
 
@@ -341,9 +367,8 @@ class PluginEntrypoint:
         stored_values: List[Dict],
     ):
         logger.info("Entered construct_form_option method")
-        stored_values_parsed = parse_obj_as(
-            List[StoredConfigurationValue], stored_values
-        )
+        stored_values_parsed = TypeAdapter(
+            List[StoredConfigurationValue]).validate_python(stored_values)
         the_function = getattr(
             self._plugin_instance,
             function_name,
@@ -353,17 +378,17 @@ class PluginEntrypoint:
             script_result = the_function(stored_value)
             if not isinstance(script_result, FormOption):
                 raise ValueError(f"Expected a FormOption from function {function_name}, got {type(script_result)}")
-            results.append(script_result.dict())
+            results.append(script_result.model_dump())
         return results
 
     def connection_form(self):
         logger.info("Entered connection_form method")
         form: List[ConnectionMethod] = self._plugin_instance.connection_form()
-        return [f.dict() for f in form]
+        return [f.model_dump() for f in form]
 
     def create_billing_events(self, session, event_request: Dict):
         logger.info("Entered create_billing_events method")
-        request = parse_obj_as(BillingEventRequest, event_request)
+        request = TypeAdapter(BillingEventRequest).validate_python(event_request)
         events: List[SnowflakeBillingEvent] = self._plugin_instance.create_billing_events(
             request
         )
@@ -396,33 +421,8 @@ class PluginEntrypoint:
                     logger.warn('Billing event creation failed due to running internally to Omnata')
                 else:
                     raise e
-        return [e.dict() for e in events]
+        return [e.model_dump() for e in events]
 
-    def get_secrets(
-        self, oauth_secret_name: Optional[str], other_secrets_name: Optional[str]
-    ) -> Dict[str, StoredConfigurationValue]:
-        connection_secrets = {}
-        import _snowflake # pylint: disable=import-error, import-outside-toplevel # type: ignore
-        if oauth_secret_name is not None:
-            connection_secrets["access_token"] = StoredConfigurationValue(
-                value=_snowflake.get_oauth_access_token(oauth_secret_name)
-            )
-        if other_secrets_name is not None:
-            try:
-                secret_string_content = _snowflake.get_generic_secret_string(
-                    other_secrets_name
-                )
-                if len(secret_string_content) > 2:
-                    other_secrets = json.loads(secret_string_content)
-                    connection_secrets = {
-                        **connection_secrets,
-                        **parse_obj_as(Dict[str, StoredConfigurationValue], other_secrets),
-                    }
-            except Exception as exception:
-                logger.error(f"Error parsing secrets content for secret {other_secrets_name}: {str(exception)}")
-                raise ValueError(f"Error parsing secrets content: {str(exception)}") from exception
-        return connection_secrets
-    
     def ngrok_post_tunnel_fields(
         self,
         connection_method: str,
@@ -434,10 +434,9 @@ class PluginEntrypoint:
         logger.info("Entered ngrok_post_tunnel_fields method")
         oauth_secret_name = normalise_nulls(oauth_secret_name)
         other_secrets_name = normalise_nulls(other_secrets_name)
-        connection_secrets = self.get_secrets(oauth_secret_name, other_secrets_name)
-        connection_parameters = parse_obj_as(
-            Dict[str, StoredConfigurationValue], connection_parameters
-        )
+        connection_secrets = get_secrets(oauth_secret_name, other_secrets_name)
+        connection_parameters = TypeAdapter(
+            Dict[str, StoredConfigurationValue]).validate_python(connection_parameters)
         parameters = ConnectionConfigurationParameters(
             connection_method=connection_method,
             connection_parameters=connection_parameters,
@@ -450,7 +449,7 @@ class PluginEntrypoint:
         script_result = the_function(parameters)
         if isinstance(script_result, List):
             if len(script_result) > 0 and isinstance(script_result[0], BaseModel):
-                script_result = [r.dict() for r in script_result]
+                script_result = [r.model_dump() for r in script_result]
         else:
             raise ValueError(f"Expected a List from function {function_name}, got {type(script_result)}")
         return script_result
@@ -465,9 +464,8 @@ class PluginEntrypoint:
         return self._plugin_instance.network_addresses(
             ConnectionConfigurationParameters(
                 connection_method=method,
-                connection_parameters=parse_obj_as(
-                    Dict[str, StoredConfigurationValue], connection_parameters
-                ),
+                connection_parameters=TypeAdapter(
+                    Dict[str, StoredConfigurationValue]).validate_python(connection_parameters),
                 connection_secrets={},
             )
         )
@@ -482,16 +480,15 @@ class PluginEntrypoint:
     ):
         logger.info("Entered connect method")
         logger.info(f"Connection parameters: {connection_parameters}")
-        connection_secrets = self.get_secrets(oauth_secret_name, other_secrets_name)
+        connection_secrets = get_secrets(oauth_secret_name, other_secrets_name)
 
         from omnata_plugin_runtime.omnata_plugin import (
             ConnectionConfigurationParameters,
         )
         parameters = ConnectionConfigurationParameters(
             connection_method=method,
-            connection_parameters=parse_obj_as(
-                Dict[str, StoredConfigurationValue], connection_parameters
-            ),
+            connection_parameters=TypeAdapter(
+                Dict[str, StoredConfigurationValue]).validate_python(connection_parameters),
             connection_secrets=connection_secrets
         )
         if oauth_secret_name is not None:
@@ -522,7 +519,7 @@ class PluginEntrypoint:
                 f"alter network rule {network_rule_name} set value_list = ({rule_values_string})"
             ).collect()
 
-        return connect_response.dict()
+        return connect_response.model_dump()
 
     def api_limits(self,
                    method:str,
@@ -530,21 +527,20 @@ class PluginEntrypoint:
                     oauth_secret_name: Optional[str],
                     other_secrets_name: Optional[str]):
         logger.info("Entered api_limits method")
-        connection_secrets = self.get_secrets(oauth_secret_name, other_secrets_name)
+        connection_secrets = get_secrets(oauth_secret_name, other_secrets_name)
         from omnata_plugin_runtime.omnata_plugin import (
             ConnectionConfigurationParameters,
         )
         connection_parameters = ConnectionConfigurationParameters(
             connection_method=method,
-            connection_parameters=parse_obj_as(
-                Dict[str, StoredConfigurationValue], connection_parameters
-            ),
+            connection_parameters=TypeAdapter(
+                Dict[str, StoredConfigurationValue]).validate_python(connection_parameters),
             connection_secrets=connection_secrets
         )
         if oauth_secret_name is not None:
             connection_parameters.access_token_secret_name = oauth_secret_name
         response: List[ApiLimits] = self._plugin_instance.api_limits(connection_parameters)
-        return [api_limit.dict() for api_limit in response]
+        return [api_limit.model_dump() for api_limit in response]
 
     def outbound_record_validator(
         self,
@@ -554,12 +550,9 @@ class PluginEntrypoint:
         source_types: Dict[str, str],
     ):
         # There's a bit of parsing here that could possibly be done outside of the handler function, but this shouldn't be too expensive
-        sync_parameters: Dict[str, StoredConfigurationValue] = parse_obj_as(
-            Dict[str, StoredConfigurationValue], sync_parameters
-        )
-        field_mappings: StoredMappingValue = parse_obj_as(
-            StoredMappingValue, field_mappings
-        )
+        sync_parameters: Dict[str, StoredConfigurationValue] = TypeAdapter(
+            Dict[str, StoredConfigurationValue]).validate_python(sync_parameters)
+        field_mappings: StoredMappingValue = TypeAdapter(StoredMappingValue).validate_python(field_mappings)
         return self._plugin_instance.outbound_record_validator(
             sync_parameters, field_mappings, transformed_record, source_types
         )
